@@ -20,6 +20,7 @@
 
 
 require_once dirname(__FILE__).'/js/configuration.host.edit.js.php';
+require_once dirname(__FILE__).'/js/common.template.edit.js.php';
 
 $widget = (new CWidget())
 	->setTitle(_('Hosts'))
@@ -31,14 +32,14 @@ if (!hasRequest('form_refresh')) {
 }
 
 $frmHost = (new CForm())
+	->setId('hostsForm')
 	->setName('hostsForm')
 	->setAttribute('aria-labeledby', ZBX_STYLE_PAGE_TITLE)
 	->addVar('form', $data['form'])
 	->addVar('clear_templates', $data['clear_templates'])
 	->addVar('flags', $data['flags'])
 	->addItem((new CVar('tls_connect', $data['tls_connect']))->removeId())
-	->addVar('tls_accept', $data['tls_accept'])
-	->setAttribute('id', 'hostForm');
+	->addVar('tls_accept', $data['tls_accept']);
 
 if ($data['hostid'] != 0) {
 	$frmHost->addVar('hostid', $data['hostid']);
@@ -51,10 +52,14 @@ $hostList = new CFormList('hostlist');
 
 // LLD rule link
 if ($data['flags'] == ZBX_FLAG_DISCOVERY_CREATED) {
-	$hostList->addRow(_('Discovered by'),
-		new CLink($data['discoveryRule']['name'],
-			(new CUrl('host_prototypes.php'))->setArgument('parent_discoveryid', $data['discoveryRule']['itemid'])
+	$hostList->addRow(_('Discovered by'), $data['discoveryRule']
+		? new CLink($data['discoveryRule']['name'],
+			(new CUrl('host_prototypes.php'))
+				->setArgument('form', 'update')
+				->setArgument('parent_discoveryid', $data['discoveryRule']['itemid'])
+				->setArgument('hostid', $data['hostDiscovery']['parent_hostid'])
 		)
+		: (new CSpan(_('Inaccessible discovery rule')))->addClass(ZBX_STYLE_GREY)
 	);
 }
 
@@ -262,7 +267,9 @@ else {
 }
 
 $hostList->addRow(_('Description'),
-	(new CTextArea('description', $data['description']))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+	(new CTextArea('description', $data['description']))
+		->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+		->setMaxlength(DB::getFieldLength('hosts', 'description'))
 );
 
 // Proxy
@@ -545,8 +552,9 @@ if ($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED) {
 		->setAttribute('style', 'width: 100%;')
 		->setHeader([_('Name'), _('Action')]);
 
+	$n = 0;
 	foreach ($data['linked_templates'] as $template) {
-		$tmplList->addVar('templates[]', $template['templateid']);
+		$tmplList->addVar('templates['.$n++.']', $template['templateid']);
 
 		if (array_key_exists($template['templateid'], $data['writable_templates'])) {
 			$template_link = (new CLink($template['name'],
@@ -580,41 +588,26 @@ if ($data['flags'] != ZBX_FLAG_DISCOVERY_CREATED) {
 		$disableids[] = $template['templateid'];
 	}
 
+	$linkedTemplateTable->addRow([
+		(new CSimpleButton(_('Add')))
+			->onClick('return PopUp("popup.generic",'.CJs::encodeJson([
+				'dstfrm' => $frmHost->getName(),
+				'dstfld1' => $frmHost->getName(),
+				'srctbl' => 'templates',
+				'srcfld1' => 'hostid',
+				'templated_hosts' => '1',
+				'popup_type' => 'templates',
+				'multiselect' => 1,
+				'disableids' => $disableids
+			]).', null, this);')
+			->addClass(ZBX_STYLE_BTN_LINK)
+	]);
+
 	$tmplList->addRow(_('Linked templates'),
 		(new CDiv($linkedTemplateTable))
 			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
 			->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
 	);
-
-	// create new linked template table
-	$new_template_table = (new CTable())
-		->addRow([
-			(new CMultiSelect([
-				'name' => 'add_templates[]',
-				'object_name' => 'templates',
-				'popup' => [
-					'parameters' => [
-						'srctbl' => 'templates',
-						'srcfld1' => 'hostid',
-						'srcfld2' => 'host',
-						'dstfrm' => $frmHost->getName(),
-						'dstfld1' => 'add_templates_',
-						'disableids' => $disableids
-					]
-				]
-			]))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-		])
-		->addRow([
-			(new CSimpleButton(_('Add')))
-				->onClick('javascript: submitFormWithParam("'.$frmHost->getName().'", "add_template", "1");')
-				->addClass(ZBX_STYLE_BTN_LINK)
-		]);
-
-	$tmplList->addRow((new CLabel(_('Link new templates'), 'add_templates__ms')),
-		(new CDiv($new_template_table))
-			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-			->setAttribute('style', 'min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
-	);
 }
 // templates for discovered hosts
 else {
