@@ -1,16 +1,16 @@
 <script type="text/x-jquery-tmpl" id="dcheckRowTPL">
 	<?= (new CRow([
 			(new CCol(
-				new CSpan('#{name}')
+				(new CDiv('#{name}'))->addClass(ZBX_STYLE_WORDWRAP)
 			))->setId('dcheckCell_#{dcheckid}'),
-			new CHorList([
+			(new CHorList([
 				(new CButton(null, _('Edit')))
 					->addClass(ZBX_STYLE_BTN_LINK)
 					->onClick("javascript: showNewCheckForm(null, '#{dcheckid}');"),
 				(new CButton(null, _('Remove')))
 					->addClass(ZBX_STYLE_BTN_LINK)
 					->onClick("javascript: removeDCheckRow('#{dcheckid}');")
-			])
+			]))->addClass(ZBX_STYLE_NOWRAP)
 		]))
 			->setId('dcheckRow_#{dcheckid}')
 			->toString()
@@ -21,7 +21,7 @@
 			(new CInput('radio', 'uniqueness_criteria', '#{dcheckid}'))
 				->addClass(ZBX_STYLE_CHECKBOX_RADIO)
 				->setId('uniqueness_criteria_#{dcheckid}'),
-			new CLabel([new CSpan(), '#{name}'], 'uniqueness_criteria_#{dcheckid}')
+			(new CLabel([new CSpan(), '#{name}'], 'uniqueness_criteria_#{dcheckid}'))->addClass(ZBX_STYLE_WORDWRAP)
 		]))
 			->setId('uniqueness_criteria_row_#{dcheckid}')
 			->toString()
@@ -33,7 +33,7 @@
 				->addClass(ZBX_STYLE_CHECKBOX_RADIO)
 				->setAttribute('data-id', '#{dcheckid}')
 				->setId('host_source_#{dcheckid}'),
-			new CLabel([new CSpan(), '#{name}'], 'host_source_#{dcheckid}')
+			(new CLabel([new CSpan(), '#{name}'], 'host_source_#{dcheckid}'))->addClass(ZBX_STYLE_WORDWRAP)
 		]))
 			->setId('host_source_row_#{dcheckid}')
 			->toString()
@@ -45,7 +45,7 @@
 				->addClass(ZBX_STYLE_CHECKBOX_RADIO)
 				->setAttribute('data-id', '#{dcheckid}')
 				->setId('name_source_#{dcheckid}'),
-			new CLabel([new CSpan(), '#{name}'], 'name_source_#{dcheckid}')
+			(new CLabel([new CSpan(), '#{name}'], 'name_source_#{dcheckid}'))->addClass(ZBX_STYLE_WORDWRAP)
 		]))
 			->setId('name_source_row_#{dcheckid}')
 			->toString()
@@ -64,24 +64,21 @@
 					(new CLabel(_('Port range'), 'ports'))->setAsteriskMark(),
 					(new CTextBox('ports'))
 						->setWidth(ZBX_TEXTAREA_SMALL_WIDTH)
-						->setAriaRequired()
-						->setAttribute('maxlength', 255),
+						->setAriaRequired(),
 					'newCheckPortsRow'
 				)
 				->addRow(
 					(new CLabel(_('SNMP community'), 'snmp_community'))->setAsteriskMark(),
 					(new CTextBox('snmp_community'))
 						->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
-						->setAriaRequired()
-						->setAttribute('maxlength', 255),
+						->setAriaRequired(),
 					'newCheckCommunityRow'
 				)
 				->addRow(
 					(new CLabel(_('Key'), 'key_'))->setAsteriskMark(),
 					(new CTextBox('key_'))
 						->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
-						->setAriaRequired()
-						->setAttribute('maxlength', 255),
+						->setAriaRequired(),
 					'newCheckKeyRow'
 				)
 				->addRow(
@@ -95,8 +92,7 @@
 				->addRow(
 					(new CLabel(_('Context name'), 'snmpv3_contextname')),
 					(new CTextBox('snmpv3_contextname'))
-						->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH)
-						->setAttribute('maxlength', 255),
+						->setWidth(ZBX_TEXTAREA_MEDIUM_WIDTH),
 					'newCheckContextRow'
 				)
 				->addRow(
@@ -152,7 +148,7 @@
 			]))
 		]))
 			->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
-			->addStyle('min-width: '.ZBX_TEXTAREA_STANDARD_WIDTH.'px')
+			->addStyle('width: '.ZBX_TEXTAREA_STANDARD_WIDTH.'px')
 	))->setId('new_check_form')
 ?>
 </script>
@@ -336,7 +332,7 @@
 				}
 
 				// update check name
-				jQuery('#dcheckCell_' + value.dcheckid + ' span').text(value['name']);
+				jQuery('#dcheckCell_' + value.dcheckid + ' .wordwrap').text(value['name']);
 			}
 
 			var availableDeviceTypes = [ZBX_SVC.agent, ZBX_SVC.snmpv1, ZBX_SVC.snmpv2, ZBX_SVC.snmpv3],
